import numpy as np
import matplotlib.pyplot as pl
from matplotlib.collections import LineCollection
from matplotlib.patches import Circle
from matplotlib.collections import PatchCollection

def plot_network(edges, node_positions, node_radii, F, ax = None, node_colors = None, scale_radii=True):

    N_nodes = len(node_radii)
    pos = node_positions
    radii = node_radii
    
    if ax is None:
        fig, ax = pl.subplots(1,1)

    if node_colors is None: 
        node_colors = np.zeros((len(node_radii),3))

    if scale_radii:
        _pos, _radii = F.scale_radial_2D(pos, radii)
    else:
        _pos = F.radial_2D(pos)
        _radii = radii

    lines = [ [ _pos[e,0], _pos[e,1] ] for e in edges ]
    lines = [ list(zip(x, y)) for x, y in lines ]

    lc = LineCollection(lines,
                        color = 'k',
                        lw = 0.5,
                        )

    ax.add_collection(lc)

    patches = []
    for n in range(N_nodes):
        x1 = _pos[n,0]
        y1 = _pos[n,1]
        r = _radii[n]
        circle = Circle((x1, y1), r, color=node_colors[n])
        patches.append(circle)

    pc = PatchCollection(patches, match_original=True)
    ax.add_collection(pc)


    ax.axis('square')
    ax.axis('off')

    return ax

if __name__ == "__main__":


    import networkx as nx
    from networkx.generators.lattice import hexagonal_lattice_graph

    N = 5

    G = hexagonal_lattice_graph(N,N+2,with_positions = True)
    G = nx.convert_node_labels_to_integers(G)

    pos = [ n[1]['pos'] for n in G.nodes(data = True) ]
    pos = np.array(pos)
    pos /= np.amax(pos)

    N_nodes = G.number_of_nodes()

    radii = np.ones(pos.shape[:1]) * 0.02


    from fisheye import fisheye

    F = fisheye(R=0.4,focus=[0.5,0.48])

    plot_network(list(G.edges()), pos, radii, F)

    pl.show()





import numpy as np
from scipy.interpolate import interp2d, RectBivariateSpline
from scipy.ndimage.filters import gaussian_filter

def apply_to_image(img_data, F, use_cartesian = False, interpolation_order = 1, gaussian_blur=False):
    """Apply a fisheye effect to image data.

    Parameters
    ----------
    img_data : numpy.ndarray
        An array containing the image data. It has to be of shape (m, n) for greyscale,
        or of shape (m, n, k) for k color channels.
    F : fisheye.fisheye
        Instance of a fisheye class with defined radius R and focus set (both in pixel
        coordinates).
    use_cartesian: bool, default = False
        Usually, a radial fisheye function is demanded, this switch enforces the cartesian
        transformation instead.
    interpolation_order : int, default = 1
        Order of interpolation for the RectBivariateSpline algorithm
    gaussian_blur : bool, default = False
        Apply a gaussian blur fliter to the transformed area.

    Returns
    -------
    new_img_data : numpy.ndarray
        The transformed data in the shape of img_data.
    """

    orig_shape = img_data.shape
    k = interpolation_order

    if len(img_data.shape) == 2:
        data = img_data.reshape((img_data.shape[0],img_data.shape[1],1))
    elif len(img_data.shape) != 3:
        raise ValueError("Wrong shape of data:", img_data.shape)
    else:
        data = img_data.copy()

    fx, fy = F.focus
    R = F.R

    x = np.arange(data.shape[0])
    y = np.arange(data.shape[1])

    if not use_cartesian:
        coords = []

        for i in range(max(0, int(fx-R)), min(data.shape[0],int(fx+R))):
            for j in range(max(0, int(fy-R)), min(data.shape[1],int(fy+R))):
                if np.sqrt((fx-i)**2 + (fy-j)**2) < R:
                    coords.append((i,j))
    else:
        coords = [ (i,j) for i in range(max(0, int(fx-R)), min(data.shape[0],int(fx+R)))\
                         for j in range(max(0, int(fy-R)), min(data.shape[1],int(fy+R))) ]

    coord_arr = np.array(coords)

    if not use_cartesian:
        inv_coords = F.inverse_radial_2D(np.array(coord_arr,dtype=float))
    else:
        inv_coords = F.inverse_cartesian(np.array(coord_arr,dtype=float))

    new_data = data.copy()

    for color in range(data.shape[2]):
        transform_function = RectBivariateSpline(x, y, data[:,:,color],kx=k,ky=k)
        transformed = transform_function(inv_coords[:,0].flatten(), inv_coords[:,1].flatten(),grid=False)
        new_data[:,:,color] = data[:,:,color]

        new_data[coord_arr[:,0], coord_arr[:,1], color] = transformed

        if gaussian_blur:
            new_data[coord_arr[:,0], coord_arr[:,1], color] = gaussian_filter(
                                                new_data[coord_arr[:,0], coord_arr[:,1], color],
                                                sigma = 2
                                                )

    return new_data.reshape(orig_shape)

if __name__ == "__main__":
    import matplotlib.pyplot as pl

    data = pl.imread('../sandbox/example.png')

    from fisheye import fisheye

    F = fisheye(300,xw=0.3,mode='default',d=2)
    F.set_focus([400,400])

    transformed = apply_to_image(data, F)

    pl.imshow(data)

    pl.figure()

    pl.imshow(transformed)

    pl.figure()
    pl.imshow(apply_to_image(data, F, use_cartesian = True))
    pl.show()

